package com.duowan.texiu.data.migration.script;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.duowan.texiu.data.migration.model.TableInfo;
import com.duowan.texiu.data.migration.model.VideoInfo;
import com.duowan.texiu.data.migration.script.base.BaseTaskWorker;
import com.duowan.texiu.data.migration.tools.*;
import com.ojia.base.exception.HttpClientException;

/**
 * @author Zhang Miaojie
 * @version v1.0
 * @date 2019/10/10
 */
public class BatchUploadStaticResourceWorker extends BaseTaskWorker {

	public static void main(String[] args) throws Exception {
		BatchUploadStaticResourceWorker batchUploadStaticResourceWorker = new BatchUploadStaticResourceWorker();
		batchUploadStaticResourceWorker.initEnv();
		batchUploadStaticResourceWorker.enter();
	}

	private void initEnv() {
		System.out.println("请选择要迁移的数据库，根据提示，输入相应的文字编号：\n" + "\t【1】特炫-texuan_home\n" + "\t【2】hi运动-hiyd-home\n"
				+ "\t【3】hi运动-hiyd-cms");
		System.out.println("请输入");
		Scanner scanner = new Scanner(System.in);
		String prefix = "";
		int i = scanner.nextInt();
		switch (i) {
		case 1:
			prefix = "texuan";
			System.out.println("您选择了【特炫-texuan_home】");
			break;
		case 2:
			prefix = "hiyd_home";
			System.out.println("您选择了【hi运动-hiyd-home】");
			break;
		case 3:
			prefix = "hiyd_cms";
			System.out.println("您选择了【hi运动-hiyd-cms】");
			break;
		default:
			System.out.println("命令不存在，请根据提示重新输入");
			initEnv();
		}
		Constants.init(prefix);
	}

	private void enter() throws Exception {
		System.out.println("请选执行的业务，根据提示，输入相应的文字编号：\n" + "\t【1】迁移静态资源\n" + "\t【2】迁移静态资源【静态资源url包含在html内容里面】\n"
				+ "\t【3】迁移静态资源【静态资源url包含在json内容里面】\n" + "\t【4】批量生成sql更新语句");
		System.out.println("请输入");
		Scanner scanner = new Scanner(System.in);
		int i = scanner.nextInt();
		switch (i) {
		case 1:
			this.migrateStaticResources();
			break;
		case 2:
			this.migrateStaticResourcesWithHtml();
			break;
		case 3:
			this.migrateStaticResourcesWithArrayJson();
			break;
		case 4:
			this.generateUpdateSql();
			break;
		default:
			System.out.println("命令不存在，请根据提示重新输入");
			enter();
		}
		Thread.sleep(Integer.MAX_VALUE);
	}

	/**
	 * 迁移静态资源
	 *
	 * @throws Exception
	 */
	private void migrateStaticResources() throws Exception {
		handle((tableInfo) -> {
			try {
				doHandleMigrateStaticResources(tableInfo);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		});
	}

	/**
	 * 迁移静态资源【静态资源url包含在html内容里面】
	 *
	 * @throws Exception
	 */
	private void migrateStaticResourcesWithHtml() throws Exception {
		handle((tableInfo) -> {
			try {
				doHandleMigrateStaticResourcesWithHtml(tableInfo);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		});
	}

	/**
	 * 迁移静态资源【静态资源url包含在json内容里面】
	 *
	 * @throws Exception
	 */
	private void migrateStaticResourcesWithArrayJson() throws Exception {
		handle((tableInfo) -> {
			try {
				doHandleMigrateStaticResourcesWithArrayJson(tableInfo);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		});
	}

	/**
	 * 生成sql更新语句
	 *
	 * @throws Exception
	 */
	private void generateUpdateSql() throws Exception {
		handle((tableInfo) -> {
			try {
				doHandleGenerateUpdateSql(tableInfo);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		});
	}

	private void generateBatchModifyValueSql() throws Exception {
		handle((tableInfo) -> {
			try {
				doHandleGenerateBatchModifyValueSql(tableInfo);
			} catch (Exception e) {
				e.printStackTrace();
			}
		});
	}

	private void doHandleMigrateStaticResourcesWithArrayJson(TableInfo tableInfo) throws SQLException {
		System.out.println(String.format("开始处理%s", tableInfo.getTableName()));

		// 读取表的所有数据，并处理相应的业务逻辑
		int total = doReadTable(tableInfo, maps -> {
			for (Map<String, Object> map : maps) {
				for (Map.Entry<String, Object> entry : map.entrySet()) {
					if (entry.getKey().equals(tableInfo.getPrimaryKey())) {
						continue;
					}
					String value = (String) entry.getValue();
					if ("{}".equals(value)) {
						continue;
					}
					JSONArray objects = JSONArray.parseArray(value);
					JSONObject content = (JSONObject) objects.get(0);
					String small = content.getString("small");
					String big = content.getString("big");

					String smallUrl = FileUtil.downloadFileWithUrl(tableInfo, entry.getKey(),
							map.get(tableInfo.getPrimaryKey()), small);
					String bigUrl = FileUtil.downloadFileWithUrl(tableInfo, entry.getKey(),
							map.get(tableInfo.getPrimaryKey()), big);

					content.put("big", bigUrl);
					content.put("small", smallUrl);

					entry.setValue(TextUtil.formatText(objects.toJSONString()));
					try {
						// 更新表数据
						String updateSql = SqlUtil.getUpdateSql(tableInfo.getTableName(), tableInfo.getPrimaryKey(),
								map);
						System.out.println(updateSql);
						DBUtil.update(updateSql);
					} catch (SQLException e) {
						e.printStackTrace();
					}
				}
			}
		});
		System.out.println(String.format("%s表处理完毕, 一共处理了%s条数据", tableInfo.getTableName(), total));
	}

	private void doHandleMigrateStaticResourcesWithHtml(TableInfo tableInfo) throws SQLException {
		System.out.println(String.format("开始处理%s", tableInfo.getTableName()));

		// 读取表的所有数据，并处理相应的业务逻辑
		int total = doReadTable(tableInfo, maps -> {
			for (Map<String, Object> map : maps) {
				for (Map.Entry<String, Object> entry : map.entrySet()) {
					if (entry.getKey().equals(tableInfo.getPrimaryKey())) {
						continue;
					}
					String value = (String) entry.getValue();
					org.jsoup.nodes.Document document = Jsoup.parse(value);
					Elements imgEls = document.select("img");

					for (Element imgEle : imgEls) {
						// 获取html内容的图片地址
						String imgSrc = imgEle.attr("src");
						String url = FileUtil.downloadFileWithUrl(tableInfo, entry.getKey(),
								map.get(tableInfo.getPrimaryKey()), imgSrc);
						imgEle.attr("src", url);
					}

					// 替换标签内容
					String str = document.toString();
					String ret = TextUtil.formatText(str);
					entry.setValue(ret);
					try {
						// 更新表数据
						String updateSql = SqlUtil.getUpdateSql(tableInfo.getTableName(), tableInfo.getPrimaryKey(),
								map);
						System.out.println(updateSql);
						DBUtil.update(updateSql);
					} catch (SQLException e) {
						e.printStackTrace();
					}
				}
			}
		});
		System.out.println(String.format("%s表处理完毕, 一共处理了%s条数据", tableInfo.getTableName(), total));
	}

	private void doHandleGenerateBatchModifyValueSql(TableInfo tableInfo) {
		SqlUtil.getBathModifyValuSql(tableInfo.getTableName(), tableInfo.getPrimaryKey(), tableInfo.getCols());
	}

	/**
	 * 生成SQL更新语句
	 */
	private void doHandleGenerateUpdateSql(TableInfo tableInfo) throws SQLException {
		// 1, 拼接查询SQL，查询tableName表的cols列
		System.out.println(String.format("开始处理=====================================%s", tableInfo.getTableName()));

		int total = doReadTable(tableInfo, maps -> {
			// 3，生成sql更新语句
			for (Map<String, Object> map : maps) {
				String updateSql = SqlUtil.getUpdateSql(tableInfo.getTableName(), tableInfo.getPrimaryKey(), map);
				// 把sql文件写入到text中
				TextUtil.saveAsFileWriter(tableInfo.getTableName(), updateSql);
			}
		});
		System.out.println(String.format("%s表处理完毕, 一共处理了%s条数据", tableInfo.getTableName(), total));
	}

	/**
	 * 静态资源文件迁移
	 *
	 * @throws SQLException
	 */
	private void doHandleMigrateStaticResources(TableInfo tableInfo) throws SQLException {
		System.out.println(String.format("开始处理%s", tableInfo.getTableName()));

		// 读取表的所有数据，并处理相应的业务逻辑
		int total = doReadTable(tableInfo, maps -> {
			for (Map<String, Object> map : maps) {
				try {
					// 下载文件 && 上传文件到阿里云服务器
					FileUtil.downloadAndUploadFile(tableInfo, map);

					// 更新表数据
					String updateSql = SqlUtil.getUpdateSql(tableInfo.getTableName(), tableInfo.getPrimaryKey(), map);
					DBUtil.update(updateSql);
				} catch (SQLException | IOException e) {
					e.printStackTrace();
				}
			}
		});
		System.out.println(String.format("%s表处理完毕, 一共处理了%s条数据", tableInfo.getTableName(), total));
	}

	/**
	 * 单独处理Template表的url字段
	 *
	 * @throws SQLException
	 */
	private void handleTemplateUrl() throws SQLException, IOException, HttpClientException {

		TableInfo tableInfo = new TableInfo();
		ArrayList<String> cols = new ArrayList<>();
		tableInfo.setTableName("template");
		tableInfo.setPrimaryKey("flag");
		cols.add("url");
		cols.add("vid");
		cols.add("flag");
		tableInfo.setCols(cols);

		String type = "yuanhua";

		String selectSql = SqlUtil.getSelectSql(tableInfo.getTableName(), tableInfo.getCols());
		System.out.println(selectSql);
		List<Map<String, Object>> maps = DBUtil.queryList(selectSql);
		for (Map<String, Object> map : maps) {
			// 根据多玩vid拿到转码后台的多玩视频地址
			String selectDwVideoSql = "select dwVid from video where id = " + map.get("vid");
			List<Map<String, Object>> maps1 = DBUtil.queryList(selectDwVideoSql);
			Map<String, Object> dwVideoMap = maps1.get(0);

			VideoInfo dwVideoInfo = HttpUtils.getDwVideoInfo((Long) dwVideoMap.get("dwVid"));
			List<VideoInfo.ResultBean.ItemsBean> items = dwVideoInfo.getResult().getItems();
			for (VideoInfo.ResultBean.ItemsBean item : items) {
				if (type.equals(item.getDefinition())) {
					// 获取转码后的视频地址，并更新到缓存
					map.put("url", item.getTranscode().getUrls().get(0));
					break;
				}
			}

			tableInfo.getCols().remove("vid");
			// 下载文件 && 上传文件
			FileUtil.downloadAndUploadFile(tableInfo, map, 1);

			// 更新表数据
			String updateSql = SqlUtil.getUpdateSql(tableInfo.getTableName(), tableInfo.getPrimaryKey(), map);
			DBUtil.update(updateSql);
			System.out.println(updateSql);
		}
		System.out.println(String.format("%s表处理完毕", tableInfo.getTableName()));
	}
}
