package com.duowan.texiu.data.migration.script.base;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.lang3.StringUtils;
import org.springframework.util.StopWatch;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.duowan.texiu.data.migration.model.TableInfo;
import com.duowan.texiu.data.migration.tools.Constants;
import com.duowan.texiu.data.migration.tools.DBUtil;
import com.duowan.texiu.data.migration.tools.SqlUtil;

/**
 * @author Zhang Miaojie
 * @version v1.0
 * @date 2019/11/26
 */
public abstract class BaseTaskWorker {

    private StopWatch stopWatch = new StopWatch();

	protected void handle(Consumer<TableInfo> consumer) throws Exception {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder bulider = dbf.newDocumentBuilder();
		Document doc = bulider
				.parse(Thread.currentThread().getContextClassLoader().getResourceAsStream(Constants.DB_CONFIG));
		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();

		// 选取table节点
		XPathExpression compile = xpath.compile("//table");

		// 获取table节点的所有节点
		NodeList nodes = (NodeList) compile.evaluate(doc, XPathConstants.NODESET);
		for (int i = 0; i < nodes.getLength(); i++) {

			NamedNodeMap attributes = nodes.item(i).getAttributes();
			String tableName = attributes.getNamedItem("name").getNodeValue().trim();
			String primaryKey = attributes.getNamedItem("primaryKey").getNodeValue().trim();
			List<String> colList = new ArrayList<>();
			NodeList childNodes = nodes.item(i).getChildNodes();

			// 遍历所有子节点，获取节点的名称与数据，将其存与tables对象的属性进行匹配并存入到该对象
			for (int j = 0; j < childNodes.getLength(); j++) {
				Node node = childNodes.item(j);
				if (StringUtils.isNotBlank(node.getTextContent())) {
					colList.add(node.getTextContent().trim());
				}
			}
			TableInfo tableInfo = new TableInfo();
			tableInfo.setPrimaryKey(primaryKey);
			tableInfo.setTableName(tableName);
			tableInfo.setCols(colList);

			// 开始处理业务
            stopWatch.start(tableName);
			consumer.accept(tableInfo);
		    stopWatch.stop();
		}
        System.out.println(stopWatch.prettyPrint());
	}

	protected int doReadTable(TableInfo tableInfo, Consumer<List<Map<String, Object>>> consumer) throws SQLException {
	    int per = 200;
		Object lastId = Integer.valueOf(14515);
		String selectSql;
		int total = 0;
		while (true) {
			selectSql = SqlUtil.getSelectSql(tableInfo.getTableName(), tableInfo.getCols(), tableInfo.getPrimaryKey(),
					per, lastId);
			System.out.println(selectSql);
			List<Map<String, Object>> maps = DBUtil.queryList(selectSql);

			if (maps.size() <= 0) {
				break;
			}
			// 处理业务
			consumer.accept(maps);
			lastId = maps.get(maps.size() - 1).get(tableInfo.getPrimaryKey());

			total += maps.size();
		}

		return total;
	}


}
