package com.duowan.texiu.data.migration.tools;

import java.io.*;
import java.net.URL;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.duowan.texiu.data.migration.model.TableInfo;

/**
 * @author Zhang Miaojie
 * @version v1.0
 * @date 2019/10/10
 */
public class FileUtil {

	public static Map<String, Object> downloadAndUploadFile(TableInfo tableInfo, Map<String, Object> map)
			throws IOException {
		return downloadAndUploadFile(tableInfo, map, 0);
	}

	public static Map<String, Object> downloadAndUploadFile(TableInfo tableInfo, Map<String, Object> map,
			int isTemplate) throws IOException {
		long vid = 0;
		for (Map.Entry<String, Object> entry : map.entrySet()) {
			if (tableInfo.getPrimaryKey().equals(entry.getKey()) || "vid".equals(entry.getKey())) {
				continue;
			}

			String url = (String) entry.getValue();
			// 如果url为空，则跳过
			if (StringUtils.isBlank(url)) {
				map.put(entry.getKey(), url);
				continue;
			}

			if (StringUtils.isBlank(getExtensionName(url))) {
				System.out.println(url);
				continue;
			}

			System.out.println("===============> " + url);
			// 下载资源到本地
			// 判断下载目录是否存在，不存在则创建目录
			File filePath = new File(Constants.BASE_FILE_PATH + File.separator + tableInfo.getTableName()
					+ File.separator + entry.getKey());
			if (!filePath.exists()) {
				boolean mkdirs = filePath.mkdirs();
				if (mkdirs) {
					System.out.println("创建文件夹成功");
				}
			}

			String downloadPath = Constants.BASE_FILE_PATH + File.separator + tableInfo.getTableName() + File.separator
					+ entry.getKey() + File.separator + map.get(tableInfo.getPrimaryKey()) + "."
					+ getExtensionName(url);

			// 下载资源文件，如果资源下载失败，则直接跳过
			if (!downloadLocalDirectory(url, downloadPath)) {
				System.out.println(String.format("文件【%s】下载失败", url));
				continue;
			}

			UploadApiResult.DataBean dataBean = upload2AliServer(downloadPath, isTemplate);
			// 调用的上传接口，返回的是一个fastdfs的一个fileid，需要拼接一个域名
			url = dataBean.getFdfsId();
			map.put(entry.getKey(), url);
			vid = dataBean.getVid();
		}
		if (isTemplate == 1) {
			map.put("vid", vid);
			map.put("transUrl", map.get("url"));
		}
		return map;
	}

	public static boolean downloadLocalDirectory(String url, String outPath) {
		if (url == null || StringUtils.isBlank(url)) {
			throw new RuntimeException("urls不能为空");
		}
		return downloadFileFromUrl(url, outPath);
	}

	/**
	 * 链接url下载静态资源
	 *
	 * @param urlAddress
	 * @param outPath
	 */
	public static boolean downloadFileFromUrl(String urlAddress, String outPath) {
		URL url;
		boolean isDownloadSuccess = false;
		try {
			url = new URL(urlAddress);
			DataInputStream dataInputStream = new DataInputStream(url.openStream());

			FileOutputStream fileOutputStream = new FileOutputStream(new File(outPath));
			ByteArrayOutputStream output = new ByteArrayOutputStream();

			byte[] buffer = new byte[1024];
			int length;

			while ((length = dataInputStream.read(buffer)) > 0) {
				output.write(buffer, 0, length);
			}
			fileOutputStream.write(output.toByteArray());
			dataInputStream.close();
			fileOutputStream.close();

			isDownloadSuccess = true;
		} catch (IOException e) {
			e.printStackTrace();
		}

		return isDownloadSuccess;
	}


	public static String downloadFileWithUrl(TableInfo tableInfo, String field, Object primaryKeyValue, String url) {
		// 下载资源到本地
		// 判断下载目录是否存在，不存在则创建目录
		File filePath = new File(Constants.BASE_FILE_PATH + File.separator + tableInfo.getTableName()
				+ File.separator + field);
		if (!filePath.exists()) {
			boolean mkdirs = filePath.mkdirs();
			if (mkdirs) {
				System.out.println("创建文件夹成功");
			}
		}

		String downloadPath = Constants.BASE_FILE_PATH + File.separator + tableInfo.getTableName()
				+ File.separator + field + File.separator + primaryKeyValue
				+ "." + FileUtil.getExtensionName(url);

		// 下载资源文件，如果资源下载失败，则直接返回原理的静态资源地址
		if (!FileUtil.downloadLocalDirectory(url, downloadPath)) {
			System.out.println(String.format("文件【%s】下载失败", url));
			return url;
		}

		UploadApiResult.DataBean dataBean = null;
		try {
			dataBean = FileUtil.upload2AliServer(downloadPath, 0);
		} catch (IOException e) {
			e.printStackTrace();
		}
		// 调用的上传接口，返回的是一个fastdfs的一个fileid，需要拼接一个域名
		return dataBean.getFdfsId();
	}

	/**
	 * 获取文件的后缀名
	 *
	 * @param filename
	 * @return
	 */
	public static String getExtensionName(String filename) {
		if ((filename != null) && (filename.length() > 0)) {
			int dot = filename.lastIndexOf('.');
			if ((dot > -1) && (dot < (filename.length() - 1))) {
				return filename.substring(dot + 1);
			}
		}
		return "";
	}

	public static UploadApiResult.DataBean upload2AliServer(String filePath, int isTemplate) throws IOException {
		if (filePath == null || StringUtils.isBlank(filePath)) {
			throw new RuntimeException("filePaths不能为空");
		}
		String result = HttpClient.uploadFile(filePath, isTemplate);
		UploadApiResult uploadApiResult = JSONObject.parseObject(result, UploadApiResult.class);
		if (0 != uploadApiResult.getCode()) {
			throw new RuntimeException("文件上传失败！");
		}
		return uploadApiResult.getData();
	}

	public static void main(String[] args) {
		System.out.println(
				getExtensionName("http://image.yy.com/ojiastoreimage/cef9276dcdfa70fb5146311e2d03bc68_stamp."));
	}
}
