package com.duowan.texiu.data.migration.tools;

import java.io.IOException;
import java.util.*;
import java.util.Map.Entry;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.http.Consts;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * HttpClient 服务类
 *
 * @author ZENG.QING.HUA DigestUtils 曾庆华
 */
public class HttpClientService {
	private static final Log log = LogFactory.getLog("HttpClientService");
	/**
	 * 定义CloseableHttpClient
	 */
	private CloseableHttpClient httpClient = HttpClients.createDefault();
	/**
	 * 定义ObjectMapper操作json
	 */
	private ObjectMapper objectMapper = new ObjectMapper();

	private HttpClientService() {

	}

	public static HttpClientService newInstance() {
		return SingletonHolder.INSTANCE;
	}

	private static class SingletonHolder {
		public static HttpClientService INSTANCE = new HttpClientService();
	}

	/**
	 * 发送get请求
	 *
	 * @throws Exception
	 */
	public HttpResponseEntity sendGet(String uri, Map<String, String> params) {
		CloseableHttpResponse response = null;
		try {
			URIBuilder uriBuilder = new URIBuilder(uri);

			if (params != null && params.size() > 0) {
				// 设置请求参数
				Set<Entry<String, String>> entrySet = params.entrySet();
				for (Entry<String, String> entry : entrySet) {
					uriBuilder.addParameter(entry.getKey(), entry.getValue());
				}
			}
			// 创建HttpGet对象
			HttpGet httpGet = new HttpGet(uriBuilder.build());

			// 使用HttpClient运行这个对象
			response = httpClient.execute(httpGet);

			String data = (response.getEntity() != null) ? EntityUtils.toString(response.getEntity(), Consts.UTF_8)
					: null;
			return new HttpResponseEntity(response.getStatusLine().getStatusCode(), data);
		} catch (Exception e) {
			throw new RuntimeException();
		} finally {
			try {
				if (response != null) {
					response.close();
				}
			} catch (IOException e) {
				throw new RuntimeException();
			}
		}
	}

	/**
	 * 利用HttpClient发送Post请求
	 *
	 * @param uri
	 *            请求URL
	 * @param params
	 *            请求参数
	 * @return 响应实体
	 */
	public HttpResponseEntity sendPost(String uri, Map<String, Object> params, boolean json) {
		return sendPost(uri, params, null, json);
	}

	/**
	 * 利用HttpClient发送Post请求
	 *
	 * @param uri
	 *            请求URL
	 * @param params
	 *            请求参数
	 * @param headerParams
	 *            请求参数
	 * @return 响应实体
	 */
	public HttpResponseEntity sendPost(String uri, Map<String, Object> params, Map<String, String> headerParams,
			boolean json) {
		CloseableHttpResponse response = null;
		try {
			// 创建HttpPost请求对象
			HttpPost httpPost = new HttpPost(uri);
			// 设置http请求头
			if (headerParams != null) {
				for (Entry<String, String> entry : headerParams.entrySet()) {
					httpPost.setHeader(new BasicHeader(entry.getKey(), entry.getValue()));
				}
			}
			// 判断是否需要设置请求参数
			if (params != null && params.size() > 0) {
				if (json) {
					// 直接传一个json数据过去
					httpPost.setEntity(
							new StringEntity(objectMapper.writeValueAsString(params), ContentType.APPLICATION_JSON));
				} else {
					// 传的是表单数据
					// 创建集合封装请求参数
					List<NameValuePair> nvpair = new ArrayList<NameValuePair>();
					Set<Entry<String, Object>> entrySet = params.entrySet();
					for (Entry<String, Object> entry : entrySet) {
						// 添加请求参数
						nvpair.add(new BasicNameValuePair(entry.getKey(), entry.getValue().toString()));
					}
					UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(nvpair, Consts.UTF_8);
					httpPost.setEntity(formEntity);
				}
			}
			// 执行请求，得到响应对象
			response = httpClient.execute(httpPost);
			// 获取响应结果
			String data = (response.getEntity() != null) ? EntityUtils.toString(response.getEntity(), Consts.UTF_8)
					: null;
			// 返回响应结果
			return new HttpResponseEntity(response.getStatusLine().getStatusCode(), data);

		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException();
		} finally {
			try {
				if (response != null) {
					response.close();
				}
			} catch (IOException e) {
				throw new RuntimeException();
			}
		}
	}

	/**
	 * 利用HttpClient发送put请求
	 *
	 * @param uri
	 *            请求地址
	 * @param params
	 *            请求参数
	 * @return 响应数据
	 */
	public HttpResponseEntity sendPut(String uri, Map<String, String> params, boolean json) {
		CloseableHttpResponse response = null;
		try {
			HttpPut httpPut = new HttpPut(uri);

			if (params != null && params.size() > 0) {
				if (json) {
					httpPut.setEntity(
							new StringEntity(objectMapper.writeValueAsString(params), ContentType.APPLICATION_JSON));
				} else {
					List<NameValuePair> nvpair = new ArrayList<NameValuePair>();
					Set<Entry<String, String>> entrySet = params.entrySet();
					for (Entry<String, String> entry : entrySet) {
						// 设置请求参数
						nvpair.add(new BasicNameValuePair(entry.getKey(), entry.getValue()));
					}
					// 设置请求参数
					UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(nvpair, Consts.UTF_8);
					httpPut.setEntity(formEntity);
				}
				ReentrantLock reentrantLock = new ReentrantLock();
				reentrantLock.lock();
			}
			response = httpClient.execute(httpPut);

			String data = (response.getEntity() != null) ? EntityUtils.toString(response.getEntity(), Consts.UTF_8)
					: null;
			return new HttpResponseEntity(response.getStatusLine().getStatusCode(), data);
		} catch (Exception e) {
			throw new RuntimeException();
		} finally {
			try {
				if (response != null) {
					response.close();
				}
			} catch (IOException e) {
				throw new RuntimeException();
			}
		}
	}

	/**
	 * 发送delete请求
	 */

	public HttpResponseEntity sendDelete(String uri, Map<String, Object> params) {
		if (params == null) {
			params = new HashMap<>();
		}
		// 利用POST请求模拟发送delete请求
		params.put("_method", "delete");
		return sendPost(uri, params, false);
	}
}
