package com.duowan.texiu.data.migration.tools;

import com.duowan.texiu.data.migration.model.TableInfo;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Zhang Miaojie
 * @version v1.0
 * @date 2019/10/10
 */
public class SqlUtil {

	/**
	 * 拼装查询SQL语句
	 *
	 * @param tableName
	 * @param cols
	 * @return
	 */
	public static String getSelectSql(String tableName, List<String> cols) {
		return getSelectSql(tableName, cols, null, null, null);
	}

	/**
	 * 拼装查询SQL语句
	 *
	 * @param tableName
	 * @param cols
	 * @return
	 */
	public static String getSelectSql(String tableName, List<String> cols, String primaryKey, Integer per,
			Object lastId) {
		if (cols == null || cols.size() == 0) {
			throw new RuntimeException("cols 不能为空");
		}
		StringBuilder sb = new StringBuilder();
		sb.append("select ");
		for (int i = 0; i < cols.size(); i++) {
			sb.append(cols.get(i));
			if (i != (cols.size() - 1)) {
				sb.append(", ");
			}
		}
		sb.append(" from ").append(tableName);
		if (per != null && lastId != null) {
			sb.append(" where ").append(primaryKey).append(" > '").append(lastId).append("' order by ").append(primaryKey)
					.append(" limit ").append(per);
		}
		sb.append(";");
		return sb.toString();
	}

	/**
	 * 拼装更新SQL语句
	 *
	 * @param tableName
	 * @param primaryKey
	 * @param mapValue
	 * @return
	 */
	public static String getUpdateSql(String tableName, String primaryKey, Map<String, Object> mapValue) {
		if (mapValue == null || mapValue.size() == 0) {
			throw new RuntimeException("illegal parameter");
		}
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append("update ").append(tableName).append(" set ");
		for (Map.Entry<String, Object> entry : mapValue.entrySet()) {
			if (primaryKey.equals(entry.getKey())) {
				continue;
			}

			Object value = entry.getValue() == null ? "" : entry.getValue();
			stringBuilder.append(entry.getKey()).append("=").append("\"").append(value).append("\"").append(",");
		}
		stringBuilder.delete(stringBuilder.length() - 1, stringBuilder.length());
		stringBuilder.append(" where ").append(primaryKey).append(" = \"").append(mapValue.get(primaryKey))
				.append("\";\n");
		return stringBuilder.toString();
	}

	/**
	 * 批量替换掉表的某个字段的值
	 *
	 * @param tableName
	 * @param primaryKey
	 * @param cols
	 * @return
	 */
	public static String getBathModifyValuSql(String tableName, String primaryKey, List<String> cols) {
		String templateSql = "UPDATE %s SET %s = REPLACE(%s, 'http://texuan-fastdfs.duowan.com', 'http://fastdfs-texuan.oxzj.com');";
		for (String col : cols) {
			if (col.equals(primaryKey)) {
				continue;
			}
			String format = String.format(templateSql, tableName, col, col);
			System.out.println(format);
		}
		return null;
	}

	public static void main(String[] args) {
		TableInfo tableInfo = new TableInfo();
		tableInfo.setTableName("user");
		tableInfo.setPrimaryKey("user_id");

		ArrayList<String> objects = new ArrayList<>();
		objects.add("user_id");
		objects.add("name");
		tableInfo.setCols(objects);

		System.out.println(getSelectSql(tableInfo.getTableName(), tableInfo.getCols()));
	}

}
