package com.duowan.texiu.data.migration;

import java.util.Objects;

/**
 * @author Zhang Miaojie
 * @version v1.0
 * @date 2019/11/4
 */
public class DistributedLock extends RedisLock {

	private final long TIME_OUT = 60000L;
	private RedisHelper redisHelper;

	@Override
	public boolean tryLock(String key) {
		return tryLock(key, TIME_OUT);
	}

	@Override
	public boolean tryLock(String key, long time) {
		long timeout = 6000L;
		long currentTime = System.currentTimeMillis();
		String expires = String.valueOf(timeout + currentTime);
		// 设置互斥量
		if (redisHelper.setNx(key, expires) > 0) {
			// 获取锁，设置超时时间
			redisHelper.explre(expires);
			return true;
		} else {
			String currentLockTime = redisHelper.get(key);
			// 检查锁是否超时
			if (Objects.nonNull(currentLockTime) && Long.parseLong(currentLockTime) < currentTime) {
				// 获取旧的锁时间并设置互斥量
				String oldLockTime = redisHelper.getSet(key, expires);
				// 旧值与当前时间比较
				if (Objects.nonNull(oldLockTime) && Objects.equals(oldLockTime, currentLockTime)) {
					// 获取锁，设置超时时间
					redisHelper.explre(expires);
					return true;
				}
			}
			return false;
		}
	}

	@Override
	public boolean unLock(String key) {
		return false;
	}
}
